﻿using InstanceFactory.BusyIndicatorSample.UI.Xaml.Popups;
using System;
using System.Threading.Tasks;
using Windows.System;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Navigation;

namespace InstanceFactory.BusyIndicatorSample
{
  /// <summary>
  /// The app's main page
  /// </summary>
  public sealed partial class MainPage : Page
  {
    #region Private Properties

    /// <summary>
    /// Gets / sets the flag indicating if a task is in progress.
    /// </summary>
    private bool IsTaskInProgress { get; set; }

    #endregion Private Properties


    #region Public Constructors

    /// <summary>
    /// Initializes a new instance of <see cref="MainPage"/>.
    /// </summary>
    public MainPage()
    {
      this.InitializeComponent();

      // Reset the flag.
      IsTaskInProgress = false;
    }

    #endregion Public Constructors


    #region Private Methods

    /// <summary>
    /// Handles the click event of an item in the list.
    /// </summary>
    /// <param name="sender">Sender of the event.</param>
    /// <param name="args">Event-Arguments.</param>
    private async void OnItemClickedAsync
      (
      object sender,
      ItemClickEventArgs args
      )
    {
      TextBlock item = args.ClickedItem as TextBlock;

      // A text block is not clicked => do nothing
      if (item == null)
      {
        return;
      }

      switch (item.Name)
      {
        case "StartIndeterminateTaskOneTextBlock":
          await PerformInIndeterminateTaskOneAsync();
          break;
        case "StartIndeterminateTaskTwoTextBlock":
          await PerformInIndeterminateTaskTwoAsync();
          break;
        default:
          return; // Unknown: do nothing
      }
    }

    /// <summary>
    /// Performs the indeterminate task one.
    /// </summary>
    /// <returns>Nothing</returns>
    private async Task PerformInIndeterminateTaskOneAsync()
    {
      // In case another task is in Progress, do nothing.
      if (IsTaskInProgress)
      {
        return;
      }

      // Start the BusyIndicator
      BusyIndicator busyIndicator = PrepareIndeterminateTask("Performing Indeterminate Task One");

      // Pretend to do something
      await Task.Delay(5000);

      // Close the BusyIndicator
      CleanUpIndeterminateTask(busyIndicator);
    }

    /// <summary>
    /// Performs the indeterminate task one.
    /// </summary>
    /// <returns>Nothing</returns>
    private async Task PerformInIndeterminateTaskTwoAsync()
    {
      // In case another task is in Progress, do nothing.
      if (IsTaskInProgress)
      {
        return;
      }

      // Start the BusyIndicator
      BusyIndicator busyIndicator = PrepareIndeterminateTask("Performing Indeterminate Task Two");

      // Pretend to do something else
      await Task.Delay(2000);

      // Close the BusyIndicator
      CleanUpIndeterminateTask(busyIndicator);
    }

    /// <summary>
    /// Processes the preparation of an indeterminate task.
    /// </summary>
    /// <param name="title">The title to be displayed by the BusyIndicator.</param>
    /// <returns>The <see cref="BusyIndicator"/>.</returns>
    private BusyIndicator PrepareIndeterminateTask
      (
      string title
      )
    {
      // Disable the app bar
      BottomAppBar.IsEnabled = false;

      // Lock the screen by starting the BusyIndicator
      BusyIndicator busyIndicator = BusyIndicator.Start(title);

      // Set the flag
      IsTaskInProgress = true;

      // return the indicator
      return (busyIndicator);
    }

    /// <summary>
    /// Performs the post-processing of an indeterminate task.
    /// </summary>
    /// <param name="busyIndicator">The <see cref="BusyIndicator"/> used to lock the UI.</param>
    private void CleanUpIndeterminateTask
      (
      BusyIndicator busyIndicator
      )
    {
      // close the BusyIndicator
      busyIndicator.Close();

      // Reset the flag
      IsTaskInProgress = false;

      // Enable the app bar
      BottomAppBar.IsEnabled = true;
    }

    /// <summary>
    /// Invoked when the grid view received a key down event.
    /// </summary>
    /// <param name="sender">Sender of the event.</param>
    /// <param name="args">Event arguments.</param>
    private async void OnGridViewKeyDown
      (
      object sender,
      KeyRoutedEventArgs args
      )
    {
      switch (args.Key)
      {
        // Start operation one when Ctrl+1 is pressed
        case VirtualKey.Number1:
          // If the Ctrl-Key is pressed too, start the operation
          if ((Window.Current.CoreWindow.GetAsyncKeyState(VirtualKey.Control) & CoreVirtualKeyStates.Down) == CoreVirtualKeyStates.Down)
          {
            args.Handled = true;
            await PerformInIndeterminateTaskOneAsync();
          }
          break;

        // Start operation two when Ctrl+2 is pressed
        case VirtualKey.Number2:
          // If the Ctrl-Key is pressed too, start the operation
          if ((Window.Current.CoreWindow.GetAsyncKeyState(VirtualKey.Control) & CoreVirtualKeyStates.Down) == CoreVirtualKeyStates.Down)
          {
            args.Handled = true;
            await PerformInIndeterminateTaskTwoAsync();
          }
          break;

        default:
          break;
      }
    }

    /// <summary>
    /// Invoked when the user tapped the more button of the app bar.
    /// </summary>
    /// <param name="sender">Sender of the event.</param>
    /// <param name="args">Event-Arguments.</param>
    private async void OnMoreOptionsAsync
      (
      object sender,
      RoutedEventArgs args
      )
    {
      // Close the app bar
      BottomAppBar.IsOpen = false;

      // Show a message dialog.
      MessageDialog dialog = new MessageDialog("You just tapped the 'More' button of the bottom app bar.", "'More' tapped");

      await dialog.ShowAsync();
    }

    #endregion Private Methods
  }
}